class CorpusList extends List {
  constructor(idOrElement, subscriberList, options={}) {
    super(idOrElement, {...CorpusList.DEFAULT_OPTIONS, ...options});
    subscriberList.push(this);
  }


  _init(corpora) {
    for (let [id, corpus] of Object.entries(corpora)) {
      this.addCorpus(corpus);
    }

    this.update();
  }


  _update(patch) {
    let item, corpusStatusElement, operation, pathArray;

    for (operation of patch) {
      /* "/corpusId/valueName" -> ["corpusId", "valueName"] */
      pathArray = operation.path.split("/").slice(1);
      switch(operation.op) {
        case "add":
          if (pathArray.includes("results")) {break;}
          this.addCorpus(operation.value);
          this.update();
          break;
        case "remove":
          this.remove("id", pathArray[0]);
          break;
        case "replace":
          item = this.get("id", pathArray[0])[0];
          switch(pathArray[1]) {
            case "status":
              corpusStatusElement = item.elm.querySelector(".status");
              corpusStatusElement.classList.remove(...Object.values(CorpusList.STATUS_COLORS));
              corpusStatusElement.classList.add(CorpusList.STATUS_COLORS[operation.value] || CorpusList.STATUS_COLORS['default']);
              corpusStatusElement.innerHTML = operation.value;
              item.values({status: operation.value});
              break;
            default:
              break;
          }
        default:
          break;
      }
    }
  }


  addCorpus(corpus) {
    let rowElement, columnElement, serviceElement, serviceIconElement, titleElement, descriptionElement, statusElement, viewElement, viewIconElement;

    // Create a row elment, where all related information get stored
    rowElement = document.createElement("tr");
    rowElement.dataset.id = corpus.id;

    // Create a column containing a service type identifying icon
    columnElement = document.createElement("td");
    serviceElement = document.createElement("a");
    serviceElement.classList.add("btn-floating", "disabled");
    serviceIconElement = document.createElement("i");
    serviceIconElement.classList.add("material-icons");
    serviceIconElement.innerText = "book";
    serviceElement.appendChild(serviceIconElement);
    columnElement.appendChild(serviceElement);
    rowElement.appendChild(columnElement);

    // Create a column containing the title and description
    columnElement = document.createElement("td");
    titleElement = document.createElement("b");
    titleElement.classList.add("title");
    titleElement.innerText = corpus.title;
    descriptionElement = document.createElement("i");
    descriptionElement.classList.add("description");
    descriptionElement.innerText = corpus.description;
    columnElement.appendChild(titleElement);
    columnElement.appendChild(document.createElement("br"));
    columnElement.appendChild(descriptionElement);
    rowElement.appendChild(columnElement);

    // Create a column containing the current status
    columnElement = document.createElement("td");
    statusElement = document.createElement("span");
    statusElement.classList.add("badge", "new", "status", CorpusList.STATUS_COLORS[corpus.status] || CorpusList.STATUS_COLORS['default']);
    statusElement.dataset.badgeCaption = "";
    statusElement.innerText = corpus.status;
    columnElement.appendChild(statusElement);
    rowElement.appendChild(columnElement);

    // Create a column containing a button leading to a details page
    columnElement = document.createElement("td");
    columnElement.classList.add("right-align");
    viewElement = document.createElement("a");
    viewElement.classList.add("waves-effect", "waves-light", "btn-small");
    viewElement.href = `/corpora/${corpus.id}`;
    viewElement.innerText = "View ";
    viewIconElement = document.createElement("i");
    viewIconElement.classList.add("material-icons", "right");
    viewIconElement.innerText = "send";
    viewElement.appendChild(viewIconElement);
    columnElement.appendChild(viewElement);
    rowElement.appendChild(columnElement);

    /*
     * Add an entry to the List.js datastructure and immediatly replace the
     * generic DOM element with our own one created above.
    */
    this.add([{description: corpus.description, id: corpus.id,
               status: corpus.status, title: corpus.title}],
             function(items) {items[0].elm = rowElement;});
  }
}
CorpusList.DEFAULT_OPTIONS = {item: `<div>
                                       <span class="description"></span>
                                       <td class="status"></td>
                                       <span class="title"></span>
                                     </div>`,
                              page: 4,
                              pagination: {innerWindow: 8, outerWindow: 1},
                              valueNames: ["description", "title", {data: ["id"]}]};
CorpusList.STATUS_COLORS = {"unprepared": "grey",
                            "preparable": "orange",
                            "preparing": "yellow",
                            "prepared": "green",
                            "start analysis": "yellow",
                            "analysing": "green",
                            "stop analysis": "red",
                            "default": "red"};
